#ifndef _MTRC_
#define _MTRC_

#include <iostream>
#include <string>
#include <vector>
#include <map>
#include <mutex>
#include <shared_mutex>
#include <atomic>

using namespace std;

namespace marcelb {

/**
 * A template class for measuring arbitrary statistics
 */
template<typename K>
class Metrics {
    mutable shared_mutex mtx;  // Mutex for concurrent read, exclusive write
    map<K, atomic<uint64_t>> counters;  // Supports any K (e.g., string, int)

public:
    /**
     * Constructor, without predefining the name of the counter
     */
    Metrics();

    /**
     * Constructor, with predefined counters from the passed object
     */
    Metrics(map<K, uint64_t> _counters);

    /**
     * Operator[] to access each measurement counter
     */
    atomic<uint64_t>& operator[](const K& key);

    /**
     * Operator++ to increment the measurement counter incrementally
     */
    Metrics& operator++(int n);

    /**
     * Method to set the counter from the passed object
     */
    void set(map<K, uint64_t> _counters);

    /**
     * A method to reset the counter
     */
    void clear();

    /**
     * A method that returns a vector of strings of all counter names
     */
    vector<K> keys() const;

    /**
     * The method returns a map<K, uint64_t> of all measurements
     */
    map<K, uint64_t> get_data() const;

    /**
     * The method returns a map<K, uint64_t> of all measurements and resets the counters
     */
    map<K, uint64_t> get_data_and_clear();

};

}

#endif