#include "../lib/metrics.hpp"

namespace marcelb {

template<typename K>
Metrics<K>::Metrics() {}

template<typename K>
Metrics<K>::Metrics(map<K, uint64_t> _counters) {
    set(_counters);
}

template<typename K>
void Metrics<K>::clear() {
    unique_lock<shared_mutex> lock(mtx);
    counters.clear();
}

template<typename K>
vector<K> Metrics<K>::keys() const {
    shared_lock<shared_mutex> lock(mtx);
    vector<K> _keys;
    for (auto& counter : counters) {
        _keys.push_back(counter.first);
    }
    return _keys;
}

template<typename K>
map<K, uint64_t> Metrics<K>::get_data() const {
    shared_lock<shared_mutex> lock(mtx);
    map<K, uint64_t> data;
    for (const auto& counter : counters) {
        data[counter.first] = counter.second.load();
    }
    return data;
}

template<typename K>
map<K, uint64_t> Metrics<K>::get_data_and_clear() {
    map<K, uint64_t> data = get_data();
    clear();
    return data;
}

template<typename K>
atomic<uint64_t>& Metrics<K>::operator[](const K& key) {
    shared_lock<shared_mutex> lock(mtx);
    return counters[key];
}

template<typename K>
Metrics<K>& Metrics<K>::operator++(int n) {
    shared_lock<shared_mutex> lock(mtx);
    // Increment all counters by n as an example
    // for (auto& counter : counters) {
    //     counter.second.fetch_add(n, memory_order_relaxed);
    // }
    n++;
    return *this;
}

template<typename K>
void Metrics<K>::set(map<K, uint64_t> _counters) {
    unique_lock<shared_mutex> lock(mtx);
    counters.clear();
    for (auto& counter : _counters) {
        counters[counter.first].store(counter.second);
    }
}

template class Metrics<string>;
template class Metrics<uint32_t>;

}